import typst
from pptx import Presentation
from pptx.util import Cm
from pathlib import Path
from PIL import Image
from io import BytesIO
import json
import jinja2
import os
import re

FILE_PATH = os.path.dirname(os.path.realpath(__file__))


def to_html(
    input, root=None, font_paths=[], output=None, start_page=1, count=None, silent=False, sys_inputs={}
):
    if not silent:
        print(f"Compiling typst source file {input}...")
    
    images = typst.compile(input, root=root, font_paths=font_paths,
                           format="svg", sys_inputs=sys_inputs)
    if type(images) is not list:
        images = [images]

    # convert bytes to string
    images = [image.decode("utf-8") for image in images]
    # replace width="[0-9\.]+pt" height="[0-9\.]+pt" with width="100%" height="100%"
    images = [
        re.sub(
            r'width="([0-9\.]+)pt" height="([0-9\.]+)pt"',
            'width="100%" height="100%"',
            image,
        )
        for image in images
    ]

    # query <pdfpc-file> from typst file
    pdfpc = json.loads(
        typst.query(
            input, "<pdfpc-file>", root=root, font_paths=font_paths, field="value",
            sys_inputs=sys_inputs
        )
    )
    if len(pdfpc) > 0:
        pdfpc = pdfpc[0]
        idx2note = {
            page["idx"]: page["note"] for page in pdfpc["pages"] if "note" in page
        }
    else:
        pdfpc = None
        idx2note = {}

    if not silent:
        print("Creating presentation...")

    # create page iterator
    if count is None:
        count = len(images)
    page_iter = range(start_page - 1, start_page + count - 1)

    result = (
        jinja2.Environment(loader=jinja2.FileSystemLoader(FILE_PATH))
        .get_template("template.html.j2")
        .render(page_iter=page_iter, images=images, idx2note=idx2note, pdfpc=pdfpc)
    )

    # save to .html file
    if output is None:
        output = Path(input).with_suffix(".html")

    # create output directory if it doesn't exist
    output_dir = Path(output).parent
    if not output_dir.exists():
        output_dir.mkdir(parents=True)

    with open(output, "w", encoding="utf8") as f:
        f.write(result)


def to_pptx(
    input,
    output=None,
    root=None,
    font_paths=[],
    start_page=1,
    count=None,
    ppi=500,
    silent=False,
    sys_inputs={},
):

    if not silent:
        print(f"Compiling typst source file {input}...")

    images = typst.compile(
        input, root=root, font_paths=font_paths, format="png", ppi=ppi,
        sys_inputs=sys_inputs
    )
    if type(images) is not list:
        images = [images]

    # query <pdfpc-file> from typst file
    pdfpc = json.loads(
        typst.query(
            input, "<pdfpc-file>", root=root, font_paths=font_paths, field="value",
            sys_inputs=sys_inputs
        )
    )
    if len(pdfpc) > 0:
        pdfpc = pdfpc[0]
        idx2note = {
            page["idx"]: page["note"] for page in pdfpc["pages"] if "note" in page
        }
    else:
        pdfpc = None
        idx2note = {}

    if not silent:
        print("Creating presentation...")

    # create pptx presentation
    prs = Presentation()
    blank_slide_layout = prs.slide_layouts[6]

    # configure presentation aspect ratio
    page = Image.open(BytesIO(images[0]))
    aspect_ratio = page.width / page.height
    prs.slide_width = int(prs.slide_height * aspect_ratio)

    # create page iterator
    if count is None:
        count = len(images)
    page_iter = range(start_page - 1, start_page + count - 1)

    # iterate over slides
    for page_no in page_iter:
        image_file = BytesIO(images[page_no])

        # add a slide
        slide = prs.slides.add_slide(blank_slide_layout)
        left = top = Cm(0)
        slide.shapes.add_picture(image_file, left, top, height=prs.slide_height)

        # add speaker notes
        if page_no in idx2note:
            notes_slide = slide.notes_slide
            text_frame = notes_slide.notes_text_frame
            text_frame.text = idx2note[page_no]

    if output is None:
        output = Path(input).with_suffix(".pptx")

    # save presentation
    prs.save(output)

    # create output directory if it doesn't exist
    output_dir = Path(output).parent
    if not output_dir.exists():
        output_dir.mkdir(parents=True)

    if not silent:
        print(f"Presentation saved to {output}")


def to_pdf(input, output=None, root=None, font_paths=[], silent=False, sys_inputs={}):
    if not silent:
        print(f"Compiling typst source file {input}...")
    if output is None:
        output = Path(input).with_suffix(".pdf")
    # create output directory if it doesn't exist
    output_dir = Path(output).parent
    if not output_dir.exists():
        output_dir.mkdir(parents=True)
    typst.compile(input, output=output, root=root, font_paths=font_paths, format="pdf",
                  sys_inputs=sys_inputs)


def to_pdfpc(input, output=None, root=None, font_paths=[], silent=False, sys_inputs={}):
    if not silent:
        print(f"Compiling typst source file {input}...")

    # save to .pdfpc file
    if output is None:
        output = Path(input).with_suffix(".pdfpc")

    # create output directory if it doesn't exist
    output_dir = Path(output).parent
    if not output_dir.exists():
        output_dir.mkdir(parents=True)

    with open(output, "w", encoding="utf8") as f:
        f.write(
            typst.query(
                input,
                "<pdfpc-file>",
                root=root,
                font_paths=font_paths,
                field="value",
                one=True,
                sys_inputs=sys_inputs,
            )
        )


if __name__ == "__main__":
    # to_pptx("example.typ")
    to_html("example.typ")



"""cli entry point for touying
"""

import argparse
import json
from . import exporter


def main():
    parser = argparse.ArgumentParser(description="CLI tool for Touying")

    subparsers = parser.add_subparsers(dest="command", required=True)

    # Subparser for the compile command
    parser_compile = subparsers.add_parser("compile", help="Compile a file")
    parser_compile.add_argument("input", help="Input file")
    parser_compile.add_argument("--output", default=None, help="Output file")
    parser_compile.add_argument(
        "--root", default=None, help="Root directory for typst file"
    )
    parser_compile.add_argument(
        "--font-paths", nargs="*", default=[], help="Paths to custom fonts"
    )
    parser_compile.add_argument(
        "--start-page", type=int, default=1, help="Page to start from"
    )
    parser_compile.add_argument(
        "--count", type=int, default=None, help="Number of pages to convert"
    )
    parser_compile.add_argument(
        "--ppi", type=int, default=500, help="Pixels per inch for PPTX format"
    )
    parser_compile.add_argument(
        "--silent", type=bool, default=False, help="Run silently"
    )
    parser_compile.add_argument(
        "--format",
        choices=["html", "pptx", "pdf", "pdfpc"],
        default="html",
        help="Output format",
    )
    parser_compile.add_argument(
        "--sys-inputs",
        default=None,
        help="JSON string to pass to typst's sys.inputs",
    )

    args = parser.parse_args()
    
    # Parse the sys_inputs JSON string if provided
    sys_inputs_dict = {}
    if args.sys_inputs:
        sys_inputs_dict = json.loads(args.sys_inputs)
        # Validate that all values are strings
        for key, value in sys_inputs_dict.items():
            if not isinstance(value, str):
                raise ValueError(f"Error in sys-inputs: Value for '{key}' must be a string, got {type(value).__name__}")

    if args.command == "compile":
        if args.format == "html":
            exporter.to_html(
                args.input,
                root=args.root,
                font_paths=args.font_paths,
                output=args.output,
                start_page=args.start_page,
                count=args.count,
                silent=args.silent,
                sys_inputs=sys_inputs_dict,
            )
        elif args.format == "pptx":
            exporter.to_pptx(
                args.input,
                output=args.output,
                root=args.root,
                font_paths=args.font_paths,
                start_page=args.start_page,
                count=args.count,
                ppi=args.ppi,
                silent=args.silent,
                sys_inputs=sys_inputs_dict,
            )
        elif args.format == "pdf":
            exporter.to_pdf(
                args.input,
                output=args.output,
                root=args.root,
                font_paths=args.font_paths,
                silent=args.silent,
                sys_inputs=sys_inputs_dict,
            )
        elif args.format == "pdfpc":
            exporter.to_pdfpc(
                args.input,
                output=args.output,
                root=args.root,
                font_paths=args.font_paths,
                silent=args.silent,
                sys_inputs=sys_inputs_dict,
            )
        else:
            raise ValueError(f"Unsupported format: {args.format}")


if __name__ == "__main__":
    main()
